/*
 * PmodISNS20.c
 *
 *  Created on: Nov 25, 2017
 *      Author: Krzysztof Chojnowski
 */

#include "stm32l4xx_hal.h"
#include "PmodISNS20.h"

SPI_HandleTypeDef pmodIsns20Spi;

void PmodISNS20_Config(void)
{
  // Configure the SPI connected to the Pmod module. Only the MISO line is required.
  pmodIsns20Spi.Instance = SPI1;
  pmodIsns20Spi.Init.Mode = SPI_MODE_MASTER;
  pmodIsns20Spi.Init.Direction = SPI_DIRECTION_2LINES_RXONLY;
  pmodIsns20Spi.Init.DataSize = SPI_DATASIZE_16BIT;
  pmodIsns20Spi.Init.CLKPolarity = SPI_POLARITY_HIGH;
  pmodIsns20Spi.Init.CLKPhase = SPI_PHASE_1EDGE;
  pmodIsns20Spi.Init.NSS = SPI_NSS_SOFT;
  pmodIsns20Spi.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_64;
  pmodIsns20Spi.Init.FirstBit = SPI_FIRSTBIT_MSB;
  pmodIsns20Spi.Init.TIMode = SPI_TIMODE_DISABLE;
  pmodIsns20Spi.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  pmodIsns20Spi.Init.NSSPMode = SPI_NSS_PULSE_DISABLE;

  HAL_SPI_Init(&pmodIsns20Spi);
}

void HAL_SPI_MspInit(SPI_HandleTypeDef *hspi)
{
  // Initialize GPIO used by the SPI2 peripheral. The CS is configured by the software (PB0 pin).
  __HAL_RCC_SPI1_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();
  __HAL_RCC_GPIOE_CLK_ENABLE();

  GPIO_InitTypeDef GPIO_InitStruct;

  GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF5_SPI1;
  GPIO_InitStruct.Pin = GPIO_PIN_1;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  GPIO_InitStruct.Pin = GPIO_PIN_14;
  HAL_GPIO_Init(GPIOE, &GPIO_InitStruct);

  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pin = GPIO_PIN_0;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_SET);
}

int32_t PmodISNS20_GetValue(void)
{
  uint16_t adcValue = 0;

  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_RESET);
  HAL_SPI_Receive(&pmodIsns20Spi, (uint8_t*)&adcValue, 1, 100);
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_SET);

  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_RESET);
  HAL_SPI_Receive(&pmodIsns20Spi, (uint8_t*)&adcValue, 1, 100);
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_SET);

  // The current value in [mA] is calculated according to the formula given in the PmodISNS20
  // reference manual.
  return (int32_t)(((int16_t)adcValue - 2048) * 1000.0 / 89.95);
}
